# Text::FreeBSD::XmlToSql
#
# Copyright 2003 by Peter Pentchev
# 
# This library is free software; you can redistribute it and/or modify
# it under the same terms as Perl itself. 
# 
# $Ringlet: perl/sys/periodicxml/Text/FreeBSD/XmlToSql/XmlToSql.pm,v 1.2 2003/07/05 03:55:15 roam Exp $

package Text::FreeBSD::XmlToSql;

use 5.005;
use strict;
use vars qw($VERSION @ISA);
@ISA = qw();

$VERSION = '0.00_01';
$VERSION = eval $VERSION;  # see L<perlmodstyle>

use Text::FreeBSD::XmlToSql::Table;

# Preloaded methods go here.

sub new()
{
	my $proto = shift;
	my $class = ref($proto) || $proto;
	my $self = {};
	my ($name, $arg);

	$self->{'STRUCT'} = undef;
	$self->{'VARS'} = { };
	$self->{'DBH'} = undef;
	bless($self, $class);
	while (defined($name = shift) && defined($arg = shift)) {
		if ($name eq 'Struct')
		{
			$self->parsestruct($arg);
		}
	}
	return $self;
}

sub parsestruct($ $)
{
	my ($self, $struct) = @_;
	
	$self->{'STRUCT'} = Text::FreeBSD::XmlToSql::Table->new(
	    Struct => $struct);
}

sub ParseXML($ $)
{
	my ($self, $tree) = @_;

	return $self->ParseXMLNode($tree, $self->{'STRUCT'}, undef);
}

sub ParseXMLNode($ $ $ $)
{
	my ($self, $tree, $table, $parent) = @_;
	my ($elname, @tree, %tbls, $nparent);
	my $q;

	return unless defined($table);
	
	if (ref($tree) ne 'ARRAY') {
		die("Array reference expected in XmlToSql::ParseXML\n");
	}
	$elname = @{$tree}[0];
	@tree = @{@{$tree}[1]}; # Yeccch!
	$q = $self->BuildQuery($tree, $table, $parent);
	$q->execute() or die("executing query for element $elname: $!: ".$self->{'DBH'}->errstr."\n");
	# FIXME: port this!
	$nparent = $self->{'DBH'}->{'mysql_insertid'};
	
	# Recurse into the tables
	%tbls = ();
	foreach my $t (@{$table->tables()}) {
		$tbls{$t->tag()} = $t;
	}
	for (my $i = 1; $i <= $#tree; $i += 2) {
		next if $tree[$i] eq '0';
		if (defined($tbls{$tree[$i]})) {
			$self->ParseXMLNode([$tree[$i], $tree[$i + 1]],
			    $tbls{$tree[$i]}, $nparent);
		}
	}
}

sub BuildQuery($ $ $)
{
	my ($self, $tree, $table, $parent) = @_;
	my ($elname, @tree);
	my ($q, $s, $idx, $fields, $f, $v, $name, $value, $i);
	my ($dbq);

	$s = '';
	$fields = $table->fields();
	$idx = 0;
	$elname = @{$tree}[0];
	@tree = @{@{$tree}[1]};
	foreach $f (keys(%{$fields})) {
		$v = $fields->{$f};
		if ($v =~ /^@(.*)$/) {
			# Get an attribute value
			$name = $1;
			die("Missing attribute $name for element $elname, SQL table ".$table->name())
			    unless (defined($tree[0]) &&
			    (ref($tree[0]) eq 'HASH') &&
			    defined($tree[0]->{$name}));
			$value = $tree[0]->{$name};
		} elsif ($v =~ /^\$(.*)$/) {
			# Get a 'special' value
			$name = $1;
			if ($name eq 'PARENT') {
				die('Bad use of $PARENT in top-level table '.
				    $table->name().", element $elname")
				    unless (defined($parent));
				$value = $parent;
			} elsif ($name eq 'TEXT') {
				$value = $tree[2];
			} elsif ($name eq 'REPDATE') {
				my %h = %{$tree[0]};
				my ($date, $time);
				if ($h{'parsed-date'} =~ /^(\d{4})(\d{2})(\d{2})$/) {
					$date = "$1-$2-$3";
				} else {
					$date = "0000-00-00";
				}
				if ($h{'parsed-time'} =~ /^(\d{2}:\d{2}:\d{2})$/) {
					$time = $1;
				} else {
					$time = "00:00:00";
				}
				$value = "$date $time";
			} elsif (exists($self->{'VARS'}->{$name})) {
				$value = $self->{'VARS'}->{$name};
			} else {
				die("Unknown special value $name in the definition of SQL table ".$table->name().", element $elname");
			}
		} else {
			# Get the child element value
			$name = $v;
			$value = undef;
			for ($i = 1; $i <= $#tree; $i += 2) {
				next unless ($name eq $tree[$i]); 
				$value = @{$tree[$i + 1]}[2]; # Gack...
				last;
			}
			die("Missing child element element $name for element $elname, SQL table ".$table->name())
			    unless (defined($value));
		}
		$s .= ", $f = ".$self->{'DBH'}->quote($value);
	}
	$s =~ s/^, //;
	$q = 'INSERT INTO '.$table->name()." SET $s";
	
	$dbq = $self->{'DBH'}->prepare($q);
	return $dbq;
}

sub setdbhandle($ $)
{
	my ($self, $dbh) = @_;

	$self->{'DBH'} = $dbh;
}

sub dbhandle($)
{
	my ($self) = @_;

	return ($self->{'DBH'});
}

sub getvar($ $)
{
	my ($self, $name) = @_;

	return ($self->{'VARS'}->{$name});
}

sub setvar($ $ $)
{
	my ($self, $name, $value) = @_;

	$self->{'VARS'}->{$name} = $value;
}

sub unsetvar($ $)
{
	my ($self, $name) = @_;

	delete($self->{'$VARS'}->{$name});
}

1;
__END__
# Below is stub documentation for your module. You'd better edit it!

=head1 NAME

Text::FreeBSD::XmlToSql - Perl extension for blah blah blah

=head1 SYNOPSIS

  use Text::FreeBSD::XmlToSql;
  blah blah blah

=head1 ABSTRACT

  This should be the abstract for Text::FreeBSD::XmlToSql.
  The abstract is used when making PPD (Perl Package Description) files.
  If you don't want an ABSTRACT you should also edit Makefile.PL to
  remove the ABSTRACT_FROM option.

=head1 DESCRIPTION

Stub documentation for Text::FreeBSD::XmlToSql, created by h2xs. It looks like the
author of the extension was negligent enough to leave the stub
unedited.

Blah blah blah.


=head1 SEE ALSO

Mention other useful documentation such as the documentation of
related modules or operating system documentation (such as man pages
in UNIX), or any relevant external documentation such as RFCs or
standards.

If you have a mailing list set up for your module, mention it here.

If you have a web site set up for your module, mention it here.

=head1 AUTHOR

Peter Pentchev, E<lt>roam@ringlet.netE<gt>

=head1 COPYRIGHT AND LICENSE

Copyright 2003 by Peter Pentchev

This library is free software; you can redistribute it and/or modify
it under the same terms as Perl itself. 

=cut
