# Text::FreeBSD::XmlToSql::Table
#
# Copyright 2003 by Peter Pentchev
# 
# This library is free software; you can redistribute it and/or modify
# it under the same terms as Perl itself. 
#
# $Ringlet: perl/sys/periodicxml/Text/FreeBSD/XmlToSql/XmlToSql/Table.pm,v 1.2 2003/07/05 03:55:16 roam Exp $

package Text::FreeBSD::XmlToSql::Table;

sub new()
{
	my $proto = shift;
	my $class = ref($proto) || $proto;
	my $self = {};

	$self->{'NAME'} = undef;
	$self->{'TAG'} = undef;
	$self->{'FIELDS'} = { };
	$self->{'IDFIELD'} = undef;
	$self->{'TABLES'} = [ ];
	bless($self, $class);
	while (defined($arg = shift)) {
		if ($arg eq 'Struct') {
			$self->parsestruct(shift);
		}
	}
	return $self;
}

sub parsestruct($ $)
{
	local ($self, $struct) = @_;

	$self->{'NAME'} = $struct->{'name'};
	$self->{'TAG'} = $struct->{'tag'};
	$self->{'FIELDS'} = $struct->{'fields'};
	$self->{'IDFIELD'} = $struct->{'id'};
	foreach $f (@{$struct->{'tables'}}) {
		push @{$self->{'TABLES'}},
		    Text::FreeBSD::XmlToSql::Table->new(Struct => $f);
	}
}

sub displaystruct($ $)
{
	my ($self, $indent) = @_;
	my $nindent = $indent."\t";
	
	print "${indent}Struct name: ".$self->{'NAME'}."\n";
	print "${indent}Struct tag: ".$self->{'TAG'}."\n";
	print "${indent}Struct fields:\n";
	print map { "${nindent}$_\t".$self->{'FIELDS'}->{$_}."\n" } keys(%{$self->{'FIELDS'}});
	print "${indent}Struct subtables:\n";
	foreach $tbl (@{$self->{'TABLES'}}) {
		$tbl->displaystruct($nindent);
	}
	print "${indent}End of struct ".$self->{'NAME'}."\n\n";
}

sub name($)
{
	my ($self) = @_;

	return $self->{'NAME'};
}

sub tag($)
{
	my ($self) = @_;

	return $self->{'TAG'};
}

sub idfield($)
{
	my ($self) = @_;

	return ($self->{'IDFIELD'});
}

sub fields($)
{
	my ($self) = @_;

	return ($self->{'FIELDS'});
}

sub tables($)
{
	my ($self) = @_;

	return ($self->{'TABLES'});
}

1;
