#!/usr/bin/perl -w
#
# $Ringlet: perl/sys/periodicxml/knobase/knobase-insert.pl,v 1.3 2003/07/05 04:06:41 roam Exp $

use strict;

use Config::Tiny;
use DBI;
use Error qw(:try);
use File::Find;
use File::Temp qw(tempfile tempdir);
use Getopt::Std;
use MIME::Parser;
use XML::Parser;
use Text::FreeBSD::XmlToSql;

my $debug = 0;

my $dbstruct = {
	'name'	=> 'reports',
	'tag'	=> 'reports',
	'id'	=> 'RepID',
	'fields'=> {
		'RepType'	=> '@type',
		'RepHost'	=> '@hostname',
		'RepDate'	=> '$REPDATE',
		'RepTextDate'	=> '@date',
		'RepName'	=> '@name',
		'RepSender'	=> '$REPSENDER',
	},
	'tables' => [
	{
		'name'	=> 'rep_mailq_qmail',
		'tag'	=> 'repitem-mailq-qmail',
		'id'	=> 'MailID',
		'fields'=> {
			'MailRep'	=> '$PARENT',
			'MailQueued'	=> 'queued',
			'MailNotProc'	=> 'not-processed',
		},
		'tables' => [
		{
			'name'	=> 'rep_mailq_qmail_msg',
			'tag'	=> 'message',
			'id'	=> 'MsgKey',
			'fields'=> {
				'MsgMail'	=> '$PARENT',
				'MsgID'		=> 'id',
				'MsgDate'	=> 'date',
				'MsgSize'	=> 'size',
				'MsgSender'	=> 'sender',
				'MsgRcpt'	=> 'rcpt',
			},
			'tables' => [ ]
		}
		]
	},
	{
		'name'	=> 'rep_disk_status',
		'tag'	=> 'repitem-status-disks',
		'id'	=> 'DiskID',
		'fields'	=> {
			'DiskRep'	=> '$PARENT',
		},
		'tables' => [
		{
			'name'	=> 'rep_disk_status_fs',
			'tag'	=> 'fs',
			'id'	=> 'FSID',
			'fields'	=> {
				'FSDisk'	=> '$PARENT',
				'FSBlocks'	=> 'blocks',
				'FSUsed'	=> 'used',
				'FSAvail'	=> 'avail',
				'FSCapacity'	=> 'cap',
				'FSMount'	=> 'mountpt',
			},
			tables => [ ]
		}
		]
	},
	{
		'name'	=> 'rep_status_net',
		'tag'	=> 'repitem-status-iface',
		'id'	=> 'NetID',
		'fields'	=> {
			'NetRep'	=> '$PARENT',
		},
		'tables' => [
		{
			'name'	=> 'rep_status_net_iface',
			'tag'	=> 'iface',
			'id'	=> 'IfID',
			'fields'	=> {
				'IfNet'		=> '$PARENT',
				'IfName'	=> '@name',
				'IfMTU'		=> 'mtu',
				'IfNetwork'	=> 'network',
				'IfAddress'	=> 'address',
				'IfInPkt'	=> 'ipkts',
				'IfInErr'	=> 'ierrs',
				'IfOutPkt'	=> 'opkts',
				'IfOutErr'	=> 'oerrs',
			},
			'tables'	=> [ ]
		}
		]
	},
	{
		'name'	=> 'rep_dumps',
		'tag'	=> 'repitem-last-dumps',
		'id'	=> 'LDumpID',
		'fields'	=> {
			'LDumpRep'	=> '$PARENT',
		},
		'tables'	=> [
		{
			'name'	=> 'rep_dumps_fs',
			'tag'	=> 'fs',
			'id'	=> 'DumpID',
			'fields'	=> {
				'DumpLast'	=> '$PARENT',
				'DumpFS'	=> '@dev',
				'DumpMount'	=> 'mountpt',
				'DumpLevel'	=> 'level',
				'DumpDate'	=> 'date',
			},
			'tables'	=> [ ]
		}
		]
	},
	{
		'name'	=> 'rep_kernlog',
		'tag'	=> 'repitem-kernlog',
		'id'	=> 'KLogID',
		'fields'	=> {
			'KLogRep'	=> '$PARENT',
		},
		'tables'	=> [
		{
			'name'	=> 'rep_kernlog_lines',
			'tag'	=> 'line',
			'id'	=> 'KernID',
			'fields'	=> {
				'KernLog'	=> '$PARENT',
				'KernText'	=> '$TEXT',
			},
			'tables'	=> [ ]
		}
		]
	},
	{
		'name'	=> 'rep_uid0',
		'tag'	=> 'repitem-uid-zero',
		'id'	=> 'ZeroID',
		'fields'	=> {
			'ZeroRep'	=> '$PARENT',
		},
		'tables'	=> [
		{
			'name'	=> 'rep_uid0_users',
			'tag'	=> 'user',
			'id'	=> 'UserKey',
			'fields'	=> {
				'UserIdx'	=> '$PARENT',
				'UserName'	=> 'username',
				'UserID'	=> 'userid',
			}
		}
		]
	},
	{
		'name'	=> 'rep_orphan',
		'tag'	=> 'repitem-orphans',
		'id'	=> 'OID',
		'fields'	=> {
			'ORep'	=> '$PARENT'
		},
		'tables'	=> [
		{
			'name'	=> 'rep_orphan_files',
			'tag'	=> 'file',
			'id'	=> 'OrphID',
			'fields'	=> {
				'OrphBatch'	=> '$PARENT',
				'OrphName'	=> '$TEXT',
			},
			'tables'	=> [ ]
		}
		]
	},
	{
		'name'	=> 'rep_pkg',
		'tag'	=> 'repitem-packages',
		'id'	=> 'PID',
		'fields'	=> {
			'PRep'	=> '$PARENT',
		},
		'tables'	=> [
		{
			'name'	=> 'rep_pkg_data',
			'tag'	=> 'package',
			'id'	=> 'PkgID',
			'fields'	=> {
				'PkgBatch'	=> '$PARENT',
				'PkgName'	=> 'name',
				'PkgReason'	=> 'reason',
				'PkgComment'	=> 'comment',
			},
			'tables'	=> [ ]
		}
		]
	},
	]
};

sub debug($)
{
	my ($msg) = @_;

	$msg =~ s/[\r\n]*$//;
	print STDERR "RDBG $msg\n" if $debug;
}

sub cleanup_remove()
{
	return if ($_ eq ".");
	&debug("-- $_\n");
	(-d($_)? rmdir: unlink) ||
		die "Removing $_: $!\n";
}

sub cleanup($)
{
	my $dir;

	return if (!defined($_[0]) || $_[0] eq '');
	if ($_[0] =~ /(.*)/) {
		$dir = $1;
	} else {
		die("Foo if failed?\n");
	}
	&debug("cleaning up $dir\n");
	finddepth(\&cleanup_remove, $dir);
	rmdir($dir);
	&debug("cleanup of $dir finished\n");
}

sub find_mime_part($ $)
{
	my ($entity, $type) = @_;
	my ($head, $ctype, $part, $e);

	&debug("fmp($entity, $type)\n");
	return undef unless (defined($entity));
	return undef unless (defined($head = $entity->head()));
	return undef unless (defined($ctype = $head->get('content-type')));
	&debug("fmp - ctype is $ctype\n");
	return $entity if ($ctype eq $type || $ctype =~ /^$type; charset=/);
	#return $entity if ($ctype eq $type);
	foreach $part ($entity->parts()) {
		&debug("fmp - part $part\n");
		if (defined($e = &find_mime_part($part, $type))) {
			&debug("fmp - returning $e!\n");
			return ($e);
		}
	}
	return undef;
}

sub debug_dump($ $)
{
	my ($el, $indent) = @_;
	my ($c, $nindent);
	
	if (!(ref $el)) {
		print $indent."Text $el\n";
	} elsif (ref $el eq 'SCALAR') {
		print $indent."Scalar ".${$el}."\n";
	} elsif (ref $el eq 'ARRAY') {
		print $indent."List:\n";
		$indent .= "\t";
		foreach $c (@{$el}) {
			&debug_dump($c, $indent);
		}
	} elsif (ref $el eq 'HASH') {
		print $indent."Hash:\n";
		$indent .= "\t";
		$nindent = $indent."\t";
		foreach $c (keys(%{$el})) {
			print $indent."Key $c\n";
			&debug_dump($el->{$c}, $nindent);
		}
	} else {
		print $indent.'Unrecognized '.ref($el)."\n";
	}
}

sub usage()
{
	die("Usage: knobase-insert [-h] [-f cfgfile] [-s cfgsection]\n".
	    "\t-f\tspecify the name of the config file to use;\n".
	    "\t-h\tdisplay this help text and exit;\n".
	    "\t-s\tspecify the name of the config file section to use.\n");
}

sub cfg_read($)
{
	my ($cfg) = @_;
	my ($fname, $section, $cfgobj, $k);

	if (defined($cfg->{'cfg_section'})) {
		$section = $cfg->{'cfg_section'};
	} else {
		$section = 'knobase';
	}
	return unless (defined($fname = $cfg->{'cfg_file'}) && -f $fname);
	return unless (defined($cfgobj = Config::Tiny->read($fname)));
	return unless (defined($cfgobj) && defined($cfgobj->{$section}));
	foreach $k (keys(%{$cfgobj->{$section}})) {
		$cfg->{$k} = $cfgobj->{$section}->{$k};
	}
}

MAIN:
{
	my %cfg = (
		'cfg_file'	=> '/etc/knobase.conf',
		'cfg_section'	=> 'knobase',
		'db_driver'	=> 'mysql',
		'db_host'	=> 'localhost',
		'db_user'	=> undef,
		'db_pass'	=> undef,
		'db_database'	=> 'periodic',
	);
	my %opts;
	my ($res, $d) = (0, undef);
	my ($tbl);
	my ($parser, $entity, $xmlent);
	my ($p, $l);
	my ($dsn, $dbh, $x);

	getopts('f:hs:', \%opts) or &usage();
	&usage() if defined($opts{'h'});
	$cfg{'cfg_file'} = $opts{'f'} if defined($opts{'f'});
	$cfg{'cfg_section'} = $opts{'s'} if defined($opts{'s'});

	&cfg_read(\%cfg);
	
	# Read a MIME message on stdin
	try
	{
		$d = tempdir();
		&debug("temp dir: $d\n");

		$x = Text::FreeBSD::XmlToSql->new(Struct => $dbstruct);
		if (defined($ENV{'SENDER'})) {
			$x->{'VARS'}->{'REPSENDER'} = $ENV{'SENDER'};
		} else {
			$x->{'VARS'}->{'REPSENDER'} = '(unknown)';
		}

		$parser = new MIME::Parser();
		$parser->output_under($d);
		if (!defined($entity = $parser->parse(\*STDIN))) {
			die("Parsing input: $!\n");
		}
		&debug("parse complete\n");
		$xmlent = &find_mime_part($entity, "text/xml");
		die ("No text/xml part!\n") if (!defined($xmlent));
		&debug("found xmlent\n");

		$p = new XML::Parser(Style => 'Tree');
		$l = $p->parse($xmlent->bodyhandle()->as_string());
		&debug("Parse result:\n");
		&debug_dump($l, '') if $debug;

		# Open an SQL connection
		$dsn = "DBI:$cfg{db_driver}:database=$cfg{db_database};host=$cfg{db_host}";
		$dbh = DBI->connect($dsn, $cfg{'db_user'}, $cfg{'db_pass'});
		if (!defined($dbh)) {
			die("Database connection failed: $!\n");
		}
		$x->setdbhandle($dbh);

		&debug("ParseXML\n");
		$x->ParseXML($l);
	}
	catch Error with
	{
		my $err = shift;
		$err =~ s/\n+$//;
		print STDERR "Processing error: $err\n";
		$res = 1;
	}
	finally
	{
		&cleanup($d);
	};
	exit($res);
}
